id: gcloud-unrestricted-ftp-access

info:
  name: Check for Unrestricted FTP Access
  author: princechaddha
  severity: high
  description: |
    Ensure that Virtual Private Cloud (VPC) firewall rules do not allow unrestricted access (0.0.0.0/0) on TCP ports 20 and 21. Restrict FTP traffic to trusted IP addresses or IP ranges to reduce the attack surface and enhance security.
  impact: |
    Allowing unrestricted access to TCP ports 20 and 21 exposes FTP servers to potential brute-force or unauthorized access attacks.
  remediation: |
    Update your VPC firewall rules to allow FTP traffic only from trusted IP addresses or ranges.
  reference:
    - https://cloud.google.com/vpc/docs/firewalls
  tags: cloud,devops,gcp,gcloud,vpc,firewall,ftp,security,networking,gcp-cloud-config

flow: |
  code(1)
  for(let projectId of iterate(template.projectIds)){
    set("projectId", projectId)
    code(2)
    for(let networkName of iterate(template.networks)){
      set("networkName", networkName)
      code(3)
      set("firewallRules", template.firewallRules)
      javascript(1)
    }
  }

self-contained: true

code:
  - engine:
      - sh
      - bash
    source: |
      gcloud projects list --format="json(projectId)"

    extractors:
      - type: json
        name: projectIds
        internal: true
        json:
          - '.[].projectId'

  - engine:
      - sh
      - bash
    source: |
      gcloud compute networks list --project $projectId --format="json(name)"

    extractors:
      - type: json
        name: networks
        internal: true
        json:
          - '.[].name'

  - engine:
      - sh
      - bash
    source: |
      gcloud compute firewall-rules list --filter network=$networkName --sort-by priority --format="json(name,disabled,direction,sourceRanges,allowed[].ports)"

    extractors:
      - type: json
        name: firewallRules
        internal: true
        json:
          - '.[]'

javascript:
  - code: |
      let firewallRules = template.firewallRules;
      if (typeof firewallRules === "string") {
        firewallRules = JSON.parse(firewallRules);
      }

      let insecureRules = [];
      for (let rule of firewallRules) {
        if (
          rule.disabled === false &&
          rule.direction === "INGRESS" &&
          Array.isArray(rule.sourceRanges) && rule.sourceRanges.includes("0.0.0.0/0") &&
          Array.isArray(rule.allowed) &&
          rule.allowed.some(allowed =>
            Array.isArray(allowed.ports) &&
            (allowed.ports.includes("20") || allowed.ports.includes("21"))
          )
        ) {
          insecureRules.push(rule.name);
        }
      }

      if (insecureRules.length > 0) {
        Export(`The firewall rules in network ${template.networkName} of project ${template.projectId} allow unrestricted FTP access: ${insecureRules.join(", ")}`);
      }

    extractors:
      - type: dsl
        dsl:
          - response
# digest: 4a0a004730450221009ff90f4d2816e588838d8b778987c09086ddbfddc2f9c7b4d131273bb177ab000220074b72e56a715b53ad84a576b95059770f0dda952d2385900db8182ab6c67675:922c64590222798bb761d5b6d8e72950